@@ -23,7 +23,7 @@ class ClockTree(TreeAnc):
     """
 
     def __init__(self,  dates=None, debug=False, real_dates=True, precision='auto',
-                 branch_length_mode='joint', *args, **kwargs):
+                 branch_length_mode='joint', use_covariation=False, *args, **kwargs):
 
         """
         ClockTree constructor
@@ -51,6 +51,14 @@ class ClockTree(TreeAnc):
             When not specified, this will default to 1 for short sequences and 2
             for long sequences with L>1e4
 
+         branch_length_mode : str
+            determines whether branch length are calculated using the 'joint' ML,
+            'marginal' ML, or branch length of the input tree ('input').
+
+         use_covariation : bool
+            determines whether root-to-tip regression accounts for covariance
+            introduced by shared ancestry.
+
          **kwargs:
             Key word argments needed to construct parent class (TreeAnc)
 
@@ -69,7 +77,7 @@ class ClockTree(TreeAnc):
         self.rel_tol_refine = ttconf.REL_TOL_REFINE
         self.branch_length_mode = branch_length_mode
         self.clock_model=None
-        self.use_covariation=True # if false, covariation will be ignored in rate estimates.
+        self.use_covariation=use_covariation # if false, covariation will be ignored in rate estimates.
         self._set_precision(precision)
         if self._assign_dates()==ttconf.ERROR:
             raise ValueError("ClockTree requires date constraints!")
@@ -192,11 +200,6 @@ class ClockTree(TreeAnc):
         ----------
         covariation : bool, optional
             account for phylogenetic covariation
-        tip_slack : float, optional
-            the excess variation/branch length associated with terminal nodes. Some
-            terminal branch length is necessary to avoid division by zero. This
-            excess branch length is measured in self.one_mutations.
-
         Returns
         -------
         TreeRegression
@@ -208,7 +211,7 @@ class ClockTree(TreeAnc):
         if covariation:
             om = self.one_mutation
             branch_variance = lambda x:((x.clock_length if hasattr(x,'clock_length') else x.mutation_length)
-                                        +(self.tip_slack*om if x.is_terminal() else 0.0))*om
+                                        +(self.tip_slack**2*om if x.is_terminal() else 0.0))*om
         else:
             branch_variance = lambda x:1.0 if x.is_terminal() else 0.0
 
@@ -221,6 +224,9 @@ class ClockTree(TreeAnc):
     def get_clock_model(self, covariation=True, slope=None):
         Treg = self.setup_TreeRegression(covariation=covariation)
         self.clock_model = Treg.regression(slope=slope)
+        if not np.isfinite(self.clock_model['slope']):
+            raise ValueError("Clock rate estimation failed. If your data lacks temporal signal, please specify the rate explicitly!")
+
         if not Treg.valid_confidence or (slope is not None):
             if 'cov' in self.clock_model:
                 self.clock_model.pop('cov')
