@@ -7,14 +7,23 @@ from .gtr import GTR
 
 class GTR_site_specific(GTR):
     """
-    Defines General-Time-Reversible model of character evolution.
+    Defines General-Time-Reversible model of character evolution that
+    allows for different models at different sites in the alignment
     """
-    def __init__(self, *args, **kwargs):
-        if 'seq_len' in kwargs:
-            self.seq_len = kwargs['seq_len']
-            kwargs.pop('seq_len')
-        else:
-            self.seq_len = 1
+    def __init__(self, seq_len=1, approximate=True, **kwargs):
+        """constructor for site specfic GTR models
+
+        Parameters
+        ----------
+        seq_len : int, optional
+            number of sites, determines dimensions of frequency vectors etc
+        approximate : bool, optional
+            use linear interpolation for exponentiated matrices to speed up calcuations
+        **kwargs
+            Description
+        """
+        self.seq_len=seq_len
+        self.approximate = approximate
         super(GTR_site_specific, self).__init__(**kwargs)
 
 
@@ -30,6 +39,7 @@ class GTR_site_specific(GTR):
             np.fill_diagonal(tmp[:,:,x], -diag_vals[:,x])
         return tmp
 
+
     def assign_rates(self, mu=1.0, pi=None, W=None):
         """
         Overwrite the GTR model given the provided data
@@ -48,35 +58,38 @@ class GTR_site_specific(GTR):
 
         """
         n = len(self.alphabet)
-        self.mu = np.copy(mu)
+        if np.isscalar(mu):
+            self._mu = mu*np.ones(self.seq_len)
+        else:
+            self._mu = np.copy(mu)
 
         if pi is not None and pi.shape[0]==n:
             self.seq_len = pi.shape[-1]
             Pi = np.copy(pi)
         else:
             if pi is not None and len(pi)!=n:
-                self.logger("length of equilibrium frequency vector does not match alphabet length", 4, warn=True)
-                self.logger("Ignoring input equilibrium frequencies", 4, warn=True)
+                raise ArgumentError("GTR_site_specific: length of equilibrium frequency vector does not match alphabet length.")
             Pi = np.ones(shape=(n,self.seq_len))
 
-        self.Pi = Pi/np.sum(Pi, axis=0)
+        self._Pi = Pi/np.sum(Pi, axis=0)
 
         if W is None or W.shape!=(n,n):
             if (W is not None) and W.shape!=(n,n):
-                self.logger("Substitution matrix size does not match alphabet size", 4, warn=True)
-                self.logger("Ignoring input substitution matrix", 4, warn=True)
-            # flow matrix
+                raise ArgumentError("GTR_site_specific: Size of substitution matrix does not match alphabet length.")
             W = np.ones((n,n))
+            np.fill_diagonal(W, 0.0)
+            np.fill_diagonal(W, - W.sum(axis=0))
         else:
             W=0.5*(np.copy(W)+np.copy(W).T)
 
         np.fill_diagonal(W,0)
         avg_pi = self.Pi.mean(axis=-1)
         average_rate = W.dot(avg_pi).dot(avg_pi)
-        self.W = W/average_rate
-        self.mu *=average_rate
+        self._W = W/average_rate
+        self._mu *=average_rate
 
         self._eig()
+        self._make_expQt_interpolator()
 
 
     @classmethod
@@ -87,15 +100,25 @@ class GTR_site_specific(GTR):
 
         Parameters
         ----------
+        L : int, optional
+            number of sites for which to generate a model
+        avg_mu : float
+           Substitution rate
+        alphabet : str
+           Alphabet name (should be standard: 'nuc', 'nuc_gap', 'aa', 'aa_gap')
+        pi_dirichlet_alpha : float, optional
+            parameter of dirichlet distribution
+        W_dirichlet_alpha : float, optional
+            parameter of dirichlet distribution
+        mu_gamma_alpha : float, optional
+            parameter of dirichlet distribution
 
-         mu : float
-            Substitution rate
-
-         alphabet : str
-            Alphabet name (should be standard: 'nuc', 'nuc_gap', 'aa', 'aa_gap')
-
-
+        Returns
+        -------
+        GTR_site_specific
+            model with randomly sampled frequencies
         """
+
         from scipy.stats import gamma
         alphabet=alphabets[alphabet]
         gtr = cls(alphabet=alphabet, seq_len=L)
@@ -124,6 +147,7 @@ class GTR_site_specific(GTR):
 
         return gtr
 
+
     @classmethod
     def custom(cls, mu=1.0, pi=None, W=None, **kwargs):
         """
@@ -157,8 +181,9 @@ class GTR_site_specific(GTR):
         gtr.assign_rates(mu=mu, pi=pi, W=W)
         return gtr
 
+
     @classmethod
-    def infer(cls, sub_ija, T_ia, root_state, pc=0.01,
+    def infer(cls, sub_ija, T_ia, root_state, pc=1.0,
               gap_limit=0.01, Nit=30, dp=1e-5, **kwargs):
         """
         Infer a GTR model by specifying the number of transitions and time spent in each
@@ -185,9 +210,8 @@ class GTR_site_specific(GTR):
          Tia :n vector
             The time spent in each character state at position a
 
-         root_state : n vector
-            The number of characters in state i in the sequence
-            of the root node.
+         root_state : np.array
+            probability that site a is in state i.
 
          pc : float
             Pseudocounts, this determines the lower cutoff on the rate when
@@ -226,21 +250,23 @@ class GTR_site_specific(GTR):
         W_ij = np.ones((q,q)) - np.eye(q)
 
         while (LA.norm(p_ia_old-p_ia)>dp) and n_iter<Nit:
+            gtr.logger(' '.join(map(str, ['GTR inference iteration',n_iter,'change:',LA.norm(p_ia_old-p_ia)])), 3)
             n_iter += 1
             p_ia_old = np.copy(p_ia)
             S_ij = np.einsum('a,ia,ja',mu_a, p_ia, T_ia)
             W_ij = (n_ij + n_ij.T + pc)/(S_ij + S_ij.T + pc)
-
+            
             avg_pi = p_ia.mean(axis=-1)
             average_rate = W_ij.dot(avg_pi).dot(avg_pi)
             W_ij = W_ij/average_rate
             mu_a *=average_rate
-
+            
             p_ia = m_ia/(mu_a*np.dot(W_ij,T_ia)+Lambda)
             p_ia = p_ia/p_ia.sum(axis=0)
-
+            
             mu_a = n_a/(pc+np.einsum('ia,ij,ja->a', p_ia, W_ij, T_ia))
 
+
         if n_iter >= Nit:
             gtr.logger('WARNING: maximum number of iterations has been reached in GTR inference',3, warn=True)
             if LA.norm(p_ia_old-p_ia) > dp:
@@ -248,8 +274,8 @@ class GTR_site_specific(GTR):
         if gtr.gap_index is not None:
             for p in range(p_ia.shape[-1]):
                 if p_ia[gtr.gap_index,p]<gap_limit:
-                    gtr.logger('The model allows for gaps which are estimated to occur at a low fraction of %1.3e'%p_ia[gtr.gap_index]+
-                           '\n\t\tthis can potentially result in artificats.'+
+                    gtr.logger('The model allows for gaps which are estimated to occur at a low fraction of %1.3e'%p_ia[gtr.gap_index,p]+
+                           '\n\t\tthis can potentially result in artifacts.'+
                            '\n\t\tgap fraction will be set to %1.4f'%gap_limit,2,warn=True)
                 p_ia[gtr.gap_index,p] = gap_limit
                 p_ia[:,p] /= p_ia[:,p].sum()
@@ -258,16 +284,6 @@ class GTR_site_specific(GTR):
         return gtr
 
 
-    def _eig_single_site(self, W, p):
-        tmpp = np.sqrt(p)
-        symQ = W*np.outer(tmpp, tmpp)
-        np.fill_diagonal(symQ, -np.sum(W*p, axis=1))
-        eigvals, eigvecs = np.linalg.eigh(symQ)
-        tmp_v = eigvecs.T*tmpp
-        one_norm = np.sum(np.abs(tmp_v), axis=1)
-        return eigvals, tmp_v.T/one_norm, (eigvecs*one_norm).T/tmpp
-
-
     def _eig(self):
         eigvals, vec, vec_inv = [], [], []
         for pi in range(self.seq_len):
@@ -288,12 +304,47 @@ class GTR_site_specific(GTR):
         self.v_inv = np.swapaxes(vec_inv, 0,-1)
 
 
-    def expQt(self, t):
-        # this is currently very slow.
-        eLambdaT = np.exp(self.mu*t*self.eigenvals)
+    def _make_expQt_interpolator(self):
+        """Function that evaluates the exponentiated substitution matrix at multiple
+        time points and constructs a linear interpolation object
+        """
+        self.rate_scale = self.average_rate().mean()
+        t_grid = (1.0/self.rate_scale)*np.concatenate((np.linspace(0,.1,11)[:-1],
+                                                     np.linspace(.1,1,21)[:-1],
+                                                     np.linspace(1,5,21)[:-1],
+                                                     np.linspace(5,10,11)))
+        stacked_expQT = np.stack([self._expQt(t) for t in t_grid], axis=0)
+
+        from scipy.interpolate import interp1d
+        self.expQt_interpolator = interp1d(t_grid, stacked_expQT, axis=0,
+                                           assume_sorted=True, copy=False, kind='linear')
+
+
+    def _expQt(self, t):
+        """Raw numerical matrix exponentiation using the diagonalized matrix.
+        This is the computational bottleneck in many simulations.
+
+        Parameters
+        ----------
+        t : float
+            time
+
+        Returns
+        -------
+        np.array
+            stack of matrices for each site
+        """
+        eLambdaT = np.exp(t*self.mu*self.eigenvals)
         return np.einsum('jia,ja,kja->ika', self.v, eLambdaT, self.v_inv)
 
 
+    def expQt(self, t):
+        if t*self.rate_scale<10 and self.approximate:
+            return self.expQt_interpolator(t)
+        else:
+            return self._expQt(t)
+
+
     def prop_t_compressed(self, seq_pair, multiplicity, t, return_log=False):
         print("NOT IMPEMENTED")
 
@@ -319,7 +370,7 @@ class GTR_site_specific(GTR):
 
         Returns
         -------
-
+`
          res : np.array
             Profile of the sequence after time t in the past.
             Shape = (L, a), where L - sequence length, a - alphabet size.
@@ -328,7 +379,7 @@ class GTR_site_specific(GTR):
         Qt = self.expQt(t)
         res = np.einsum('ai,ija->aj', profile, Qt)
 
-        return np.log(res) if return_log else res
+        return  np.log(np.maximum(ttconf.TINY_NUMBER,res)) if return_log else np.maximum(0,res)
 
 
     def evolve(self, profile, t, return_log=False):
@@ -418,4 +469,8 @@ class GTR_site_specific(GTR):
 
 
     def average_rate(self):
-        return np.einsum('a,ia,ij,ja->a',self.mu, self.Pi, self.W, self.Pi)
+        if self.Pi.shape[1]>1:
+            return np.einsum('a,ia,ij,ja->a',self.mu, self.Pi, self.W, self.Pi)
+        else:
+            return self.mu*np.einsum('ia,ij,ja->a',self.Pi, self.W, self.Pi)
+
