@@ -6,8 +6,9 @@ from treetime import config as ttconf
 from .utils import tree_layout
 from .clock_tree import ClockTree
 
-rerooting_mechanisms = ["min_dev", "min_dev_ML", "best", "least-squares", "ML"]
-deprecated_rerooting_mechanisms = {"residual":"least-squares", "res":"least-squares"}
+rerooting_mechanisms = ["min_dev", "best", "least-squares"]
+deprecated_rerooting_mechanisms = {"residual":"least-squares", "res":"least-squares",
+                                   "min_dev_ML": "min_dev", "ML":"least-squares"}
 
 class TreeTime(ClockTree):
     """
@@ -35,7 +36,7 @@ class TreeTime(ClockTree):
     def run(self, root=None, infer_gtr=True, relaxed_clock=None, n_iqd = None,
             resolve_polytomies=True, max_iter=0, Tc=None, fixed_clock_rate=None,
             time_marginal=False, sequence_marginal=False, branch_length_mode='auto',
-            vary_rate=False, use_covariation=True, **kwargs):
+            vary_rate=False, use_covariation=False, **kwargs):
 
         """
         Run TreeTime reconstruction. Based on the input parameters, it divides
@@ -103,11 +104,11 @@ class TreeTime(ClockTree):
             otherwise this standard deviation is estimated from the root-to-tip regression
 
         use_covariation : bool, optional
-            default True, if False, rate estimates will be performed using simple
+            default False, if False, rate estimates will be performed using simple
             regression ignoring phylogenetic covaration between nodes.
 
         **kwargs
-           Keyword arguments needed by the dowstream functions
+           Keyword arguments needed by the downstream functions
 
 
         Returns
@@ -148,8 +149,8 @@ class TreeTime(ClockTree):
                 self.infer_ancestral_sequences(infer_gtr=infer_gtr, **seq_kwargs)
                 self.prune_short_branches()
         else:
-            self.optimize_sequences_and_branch_length(infer_gtr=infer_gtr,
-                                                      max_iter=1, prune_short=True, **seq_kwargs)
+            self.optimize_tree(infer_gtr=infer_gtr,
+                               max_iter=1, prune_short=True, **seq_kwargs)
         avg_root_to_tip = np.mean([x.dist2root for x in self.tree.get_terminals()])
 
         # optionally reroot the tree either by oldest, best regression or with a specific leaf
@@ -159,18 +160,17 @@ class TreeTime(ClockTree):
             else:
                 plot_rtt=False
             reroot_mechanism = 'least-squares' if root=='clock_filter' else root
-            if self.clock_filter(reroot=reroot_mechanism, n_iqd=n_iqd, plot=plot_rtt)==ttconf.ERROR:
+            if self.clock_filter(reroot=reroot_mechanism, n_iqd=n_iqd, plot=plot_rtt, fixed_clock_rate=fixed_clock_rate)==ttconf.ERROR:
                 return ttconf.ERROR
         elif root is not None:
-            if self.reroot(root=root)==ttconf.ERROR:
+            if self.reroot(root=root, clock_rate=fixed_clock_rate)==ttconf.ERROR:
                 return ttconf.ERROR
 
         if self.branch_length_mode=='input':
             if self.aln:
                 self.infer_ancestral_sequences(**seq_kwargs)
         else:
-            self.optimize_sequences_and_branch_length(max_iter=1, prune_short=False,
-                                                      **seq_kwargs)
+            self.optimize_tree(max_iter=1, prune_short=False,**seq_kwargs)
 
         # infer time tree and optionally resolve polytomies
         self.logger("###TreeTime.run: INITIAL ROUND",0)
@@ -181,7 +181,7 @@ class TreeTime(ClockTree):
         self.LH =[[seq_LH, self.tree.positional_joint_LH, 0]]
 
         if root is not None and max_iter:
-            if self.reroot(root='least-squares' if root=='clock_filter' else root)==ttconf.ERROR:
+            if self.reroot(root='least-squares' if root=='clock_filter' else root, clock_rate=fixed_clock_rate)==ttconf.ERROR:
                 return ttconf.ERROR
 
         # iteratively reconstruct ancestral sequences and re-infer
@@ -213,8 +213,7 @@ class TreeTime(ClockTree):
                 if n_resolved:
                     self.prepare_tree()
                     if self.branch_length_mode!='input': # otherwise reoptimize branch length while preserving branches without mutations
-                        self.optimize_sequences_and_branch_length(prune_short=False,
-                                                          max_iter=0, **seq_kwargs)
+                        self.optimize_tree(prune_short=False, max_iter=0, **seq_kwargs)
 
                     need_new_time_tree = True
 
@@ -297,7 +296,7 @@ class TreeTime(ClockTree):
             self.branch_length_mode = 'input'
 
 
-    def clock_filter(self, reroot='least-squares', n_iqd=None, plot=False):
+    def clock_filter(self, reroot='least-squares', n_iqd=None, plot=False, fixed_clock_rate=None):
         '''
         Labels outlier branches that don't seem to follow a molecular clock
         and excludes them from subsequent molecular clock estimation and
@@ -321,15 +320,15 @@ class TreeTime(ClockTree):
         '''
         if n_iqd is None:
             n_iqd = ttconf.NIQD
+        if type(reroot) is list and len(reroot)==1:
+            reroot=str(reroot[0])
 
         terminals = self.tree.get_terminals()
         if reroot:
-            if reroot.startswith("ML"):
-                self.logger("TreeTime.ClockFilter: filtering with covariance aware methods is not recommended.", 0, warn=True)
-            if self.reroot(root='least-squares' if reroot=='best' else reroot)==ttconf.ERROR:
+            if self.reroot(root='least-squares' if reroot=='best' else reroot, covariation=False, clock_rate=fixed_clock_rate)==ttconf.ERROR:
                 return ttconf.ERROR
         else:
-            self.get_clock_model(covariation=False)
+            self.get_clock_model(covariation=False, slope=fixed_clock_rate)
 
         clock_rate = self.clock_model['slope']
         icpt = self.clock_model['intercept']
@@ -348,7 +347,7 @@ class TreeTime(ClockTree):
                 node.bad_branch=False
 
         # redo root estimation after outlier removal
-        if reroot and self.reroot(root=reroot)==ttconf.ERROR:
+        if reroot and self.reroot(root=reroot, clock_rate=fixed_clock_rate)==ttconf.ERROR:
                 return ttconf.ERROR
 
         if plot:
@@ -377,11 +376,11 @@ class TreeTime(ClockTree):
             cf = self.clock_model['valid_confidence']
         else:
             cf = False
-        Treg.clock_plot(ax=ax, add_internal=add_internal, confidence=cf, n_sigma=2,
+        Treg.clock_plot(ax=ax, add_internal=add_internal, confidence=cf, n_sigma=1,
                         regression=self.clock_model)
 
 
-    def reroot(self, root='least-squares', force_positive=True):
+    def reroot(self, root='least-squares', force_positive=True, covariation=None, clock_rate=None):
         """
         Find best root and re-root the tree to the new root
 
@@ -391,9 +390,11 @@ class TreeTime(ClockTree):
          root : str
             Which method should be used to find the best root. Available methods are:
 
-            :code:`best`, `least-squares`, `ML` - minimize squared residual or likelihood of root-to-tip regression
+            :code:`best`, `least-squares` - minimize squared residual or likelihood of root-to-tip regression
 
-            :code:`oldest` - choose the oldest node
+            :code:`min_dev` - minimize variation of root-to-tip distance
+
+            :code:`oldest` - reroot on the oldest node
 
             :code:`<node_name>` - reroot to the node with name :code:`<node_name>`
 
@@ -401,21 +402,35 @@ class TreeTime(ClockTree):
 
           force_positive : bool
             only consider positive rates when searching for the optimal root
+
+          covariation : bool
+             account for covariation in root-to-tip regression
         """
+        if type(root) is list and len(root)==1:
+            root=str(root[0])
+
         if root=='best':
             root='least-squares'
 
-        self.logger("TreeTime.reroot: with method or node: %s"%root,1)
+        use_cov = self.use_covariation if covariation is None else covariation
+        slope = 0.0 if type(root)==str and root.startswith('min_dev') else clock_rate
+
+        self.logger("TreeTime.reroot: with method or node: %s"%root,0)
         for n in self.tree.find_clades():
             n.branch_length=n.mutation_length
 
-        if root in rerooting_mechanisms or root in deprecated_rerooting_mechanisms:
+        if (type(root) is str) and \
+           (root in rerooting_mechanisms or root in deprecated_rerooting_mechanisms):
             if root in deprecated_rerooting_mechanisms:
+                if "ML" in root:
+                    use_cov=True
                 self.logger('TreeTime.reroot: rerooting mechanisms %s has been renamed to %s'
                              %(root, deprecated_rerooting_mechanisms[root]), 1, warn=True)
                 root = deprecated_rerooting_mechanisms[root]
 
-            new_root = self._find_best_root(covariation='ML' in root,
+            self.logger("TreeTime.reroot: rerooting will %s covariance and shared ancestry."%("account for" if use_cov else "ignore"),0)
+            new_root = self._find_best_root(covariation=use_cov,
+                                            slope = slope,
                                             force_positive=force_positive and (not root.startswith('min_dev')))
         else:
             if isinstance(root,Phylo.BaseTree.Clade):
@@ -436,7 +451,7 @@ class TreeTime(ClockTree):
             #(Without outgroup_branch_length, gives a trifurcating root, but this will mean
             #mutations may have to occur multiple times.)
             self.tree.root_with_outgroup(new_root, outgroup_branch_length=new_root.branch_length/2)
-            self.get_clock_model(covariation=self.use_covariation)
+            self.get_clock_model(covariation=use_cov, slope = slope)
 
 
         if new_root == ttconf.ERROR:
@@ -461,7 +476,7 @@ class TreeTime(ClockTree):
                 n.clock_length = n.branch_length
         self.prepare_tree()
 
-        self.get_clock_model(covariation=(('ML' in root) and self.use_covariation))
+        self.get_clock_model(covariation=self.use_covariation, slope=slope)
 
         return ttconf.SUCCESS
 
@@ -754,7 +769,7 @@ class TreeTime(ClockTree):
 ### rerooting
 ###############################################################################
 
-    def _find_best_root(self, covariation=True, force_positive=True, **kwarks):
+    def _find_best_root(self, covariation=True, force_positive=True, slope=None, **kwarks):
         '''
         Determine the node that, when the tree is rooted on this node, results
         in the best regression of temporal constraints and root to tip distances.
@@ -776,7 +791,7 @@ class TreeTime(ClockTree):
             n.branch_length=n.mutation_length
         self.logger("TreeTime._find_best_root: searching for the best root position...",2)
         Treg = self.setup_TreeRegression(covariation=covariation)
-        return Treg.optimal_reroot(force_positive=force_positive)['node']
+        return Treg.optimal_reroot(force_positive=force_positive, slope=slope)['node']
 
 
 def plot_vs_years(tt, step = None, ax=None, confidence=None, ticks=True, **kwargs):
