@@ -472,9 +472,11 @@ def timetree(params):
 
     dates = utils.parse_dates(params.dates)
     if len(dates)==0:
+        print("No valid dates -- exiting.")
         return 1
 
     if assure_tree(params, tmp_dir='timetree_tmp'):
+        print("No tree -- exiting.")
         return 1
 
     outdir = get_outdir(params, '_treetime')
@@ -505,7 +507,7 @@ def timetree(params):
     myTree = TreeTime(dates=dates, tree=params.tree, ref=ref,
                       aln=aln, gtr=gtr, seq_len=params.sequence_length,
                       verbose=params.verbose)
-
+    myTree.tip_slack=params.tip_slack
     if not myTree.one_mutation:
         print("TreeTime setup failed, exiting")
         return 1
@@ -520,23 +522,37 @@ def timetree(params):
             coalescent = params.coalescent
         else:
             print("unknown coalescent model specification, has to be either "
-                  "a float, 'opt', 'const' or 'skyline'")
-            coalescent = None
+                  "a float, 'opt', 'const' or 'skyline' -- exiting")
+            return 1
 
-    vary_rate = params.confidence
-    if params.clock_std_dev and params.clock_rate:
-        vary_rate = params.clock_std_dev
+    # determine whether confidence intervals are to be computed and how the
+    # uncertainty in the rate estimate should be treated
+    calc_confidence = params.confidence
+    if params.clock_std_dev:
+        vary_rate = params.clock_std_dev if calc_confidence else False
+    elif params.confidence and params.covariation:
+        vary_rate = True
+    elif params.confidence:
+        print("\nOutside of covariation aware mode TreeTime cannot estimate confidence intervals "
+                "without specified standard deviation of the clock rate. \nPlease specify '--clock-std-dev' "
+                "or rerun with '--covariation'. \nWill proceed without confidence estimation")
+        vary_rate = False
+        calc_confidence = False
+    else:
+        vary_rate = False
 
+    # RUN
     root = None if params.keep_root else params.reroot
     success = myTree.run(root=root, relaxed_clock=relaxed_clock_params,
                resolve_polytomies=(not params.keep_polytomies),
                Tc=coalescent, max_iter=params.max_iter,
                fixed_clock_rate=params.clock_rate,
                n_iqd=params.clock_filter,
-               time_marginal="assign" if params.confidence else False,
+               time_marginal="assign" if calc_confidence else False,
                vary_rate = vary_rate,
                branch_length_mode = branch_length_mode,
-               fixed_pi=fixed_pi)
+               fixed_pi=fixed_pi,
+               use_covariation = params.covariation)
     if success==ttconf.ERROR: # if TreeTime.run failed, exit
         print("\nTreeTime run FAILED: please check above for errors and/or rerun with --verbose 4.\n")
         return 1
@@ -545,9 +561,16 @@ def timetree(params):
     ### OUTPUT and saving of results
     ###########################################################################
     if infer_gtr:
-        print('\nInferred GTR model:')
+        fname = outdir+'sequence_evolution_model.txt'
+        with open(fname, 'w') as ofile:
+            ofile.write(str(myTree.gtr)+'\n')
+        print('\nInferred sequence evolution model (saved as %s):'%fname)
         print(myTree.gtr)
 
+    fname = outdir+'molecular_clock.txt'
+    with open(fname, 'w') as ofile:
+        ofile.write(str(myTree.date2dist)+'\n')
+    print('\nInferred sequence evolution model (saved as %s):'%fname)
     print(myTree.date2dist)
 
     basename = get_basename(params, outdir)
@@ -588,7 +611,7 @@ def timetree(params):
                 fh.write("%s\t%1.3e\t%1.3e\t%1.3e\t%1.2f\n"%(n.name, n.clock_length, n.mutation_length, myTree.date2dist.clock_rate*g, g))
 
     export_sequences_and_tree(myTree, basename, is_vcf, params.zero_based,
-                              timetree=True, confidence=params.confidence)
+                              timetree=True, confidence=calc_confidence)
 
     return 0
 
@@ -624,8 +647,11 @@ def ancestral_reconstruction(params):
     ###########################################################################
     ### OUTPUT and saving of results
     ###########################################################################
-    if params.gtr=="infer":
-        print('\nInferred GTR model:')
+    if params.gtr=='infer':
+        fname = outdir+'/sequence_evolution_model.txt'
+        with open(fname, 'w') as ofile:
+            ofile.write(str(treeanc.gtr)+'\n')
+        print('\nInferred sequence evolution model (saved as %s):'%fname)
         print(treeanc.gtr)
 
     export_sequences_and_tree(treeanc, basename, is_vcf, params.zero_based,
@@ -633,6 +659,72 @@ def ancestral_reconstruction(params):
 
     return 0
 
+def reconstruct_discrete_traits(tree, traits, missing_data='?', pc=1.0, sampling_bias_correction=None,
+                                weights=None, verbose=0):
+    unique_states = sorted(set(traits.values()))
+    nc = len(unique_states)
+    if nc>180:
+        print("mugration: can't have more than 180 states!", file=sys.stderr)
+        return 1
+    elif nc<2:
+        print("mugration: only one or zero states found -- this doesn't make any sense", file=sys.stderr)
+        return 1
+
+    ###########################################################################
+    ### make a single character alphabet that maps to discrete states
+    ###########################################################################
+    alphabet = [chr(65+i) for i,state in enumerate(unique_states)]
+    missing_char = chr(65+nc)
+    letter_to_state = {a:unique_states[i] for i,a in enumerate(alphabet)}
+    letter_to_state[missing_char]=missing_data
+    reverse_alphabet = {v:k for k,v in letter_to_state.items()}
+
+    ###########################################################################
+    ### construct gtr model
+    ###########################################################################
+    if type(weights)==str:
+        tmp_weights = pd.read_csv(weights, sep='\t' if weights[-3:]=='tsv' else ',',
+                             skipinitialspace=True)
+        weights = {row[0]:row[1] for ri,row in tmp_weights.iterrows()}
+        mean_weight = np.mean(list(weights.values()))
+        weights = np.array([weights[c] if c in weights else mean_weight for c in unique_states], dtype=float)
+        weights/=weights.sum()
+    else:
+        weights = None
+
+    # set up dummy matrix
+    W = np.ones((nc,nc), dtype=float)
+
+    mugration_GTR = GTR.custom(pi = weights, W=W, alphabet = np.array(alphabet))
+    mugration_GTR.profile_map[missing_char] = np.ones(nc)
+    mugration_GTR.ambiguous=missing_char
+
+    ###########################################################################
+    ### set up treeanc
+    ###########################################################################
+    treeanc = TreeAnc(tree, gtr=mugration_GTR, verbose=verbose,
+                      convert_upper=False, one_mutation=0.001)
+    treeanc.use_mutation_length = False
+    pseudo_seqs = [SeqRecord(id=n.name,name=n.name,
+                   seq=Seq(reverse_alphabet[traits[n.name]]
+                           if n.name in traits else missing_char))
+                   for n in treeanc.tree.get_terminals()]
+    treeanc.aln = MultipleSeqAlignment(pseudo_seqs)
+
+    ndiff = treeanc.infer_ancestral_sequences(method='ml', infer_gtr=True,
+            store_compressed=False, pc=pc, marginal=True, normalized_rate=False,
+            fixed_pi=weights)
+
+    if ndiff==ttconf.ERROR: # if reconstruction failed, exit
+        return 1
+
+    if sampling_bias_correction:
+        treeanc.gtr.mu *= sampling_bias_correction
+        treeanc.infer_ancestral_sequences(infer_gtr=False, store_compressed=False,
+                                     marginal=True, normalized_rate=False)
+    return treeanc, letter_to_state, reverse_alphabet
+
+
 def mugration(params):
     """
     implementing treetime mugration
@@ -664,63 +756,11 @@ def mugration(params):
 
     leaf_to_attr = {x[taxon_name]:x[attr] for xi, x in states.iterrows()
                     if x[attr]!=params.missing_data}
-    unique_states = sorted(set(leaf_to_attr.values()))
-    nc = len(unique_states)
-    if nc>180:
-        print("mugration: can't have more than 180 states!", file=sys.stderr)
-        return 1
-    elif nc<2:
-        print("mugration: only one or zero states found -- this doesn't make any sense", file=sys.stderr)
-        return 1
-
-    ###########################################################################
-    ### make a single character alphabet that maps to discrete states
-    ###########################################################################
-    alphabet = [chr(65+i) for i,state in enumerate(unique_states)]
-    missing_char = chr(65+nc)
-    letter_to_state = {a:unique_states[i] for i,a in enumerate(alphabet)}
-    letter_to_state[missing_char]=params.missing_data
-    reverse_alphabet = {v:k for k,v in letter_to_state.items()}
-
-    ###########################################################################
-    ### construct gtr model
-    ###########################################################################
-    if params.weights:
-        params.infer_gtr = True
-        tmp_weights = pd.read_csv(params.weights, sep='\t' if params.states[-3:]=='tsv' else ',',
-                             skipinitialspace=True)
-        weights = {row[0]:row[1] for ri,row in tmp_weights.iterrows()}
-        mean_weight = np.mean(list(weights.values()))
-        weights = np.array([weights[c] if c in weights else mean_weight for c in unique_states], dtype=float)
-        weights/=weights.sum()
-    else:
-        weights = np.ones(nc, dtype=float)/nc
-
-    # set up dummy matrix
-    W = np.ones((nc,nc), dtype=float)
-
-    mugration_GTR = GTR.custom(pi = weights, W=W, alphabet = np.array(alphabet))
-    mugration_GTR.profile_map[missing_char] = np.ones(nc)
-    mugration_GTR.ambiguous=missing_char
-
-    ###########################################################################
-    ### set up treeanc
-    ###########################################################################
-    treeanc = TreeAnc(params.tree, gtr=mugration_GTR, verbose=params.verbose,
-                      convert_upper=False, one_mutation=0.001)
-    pseudo_seqs = [SeqRecord(id=n.name,name=n.name,
-                   seq=Seq(reverse_alphabet[leaf_to_attr[n.name]]
-                           if n.name in leaf_to_attr else missing_char))
-                   for n in treeanc.tree.get_terminals()]
-    treeanc.aln = MultipleSeqAlignment(pseudo_seqs)
-
-    ndiff = treeanc.infer_ancestral_sequences(method='ml', infer_gtr=True,
-            store_compressed=False, pc=params.pc, marginal=True, normalized_rate=False,
-            fixed_pi=weights if params.weights else None)
-    if ndiff==ttconf.ERROR: # if reconstruction failed, exit
-        return 1
 
+    mug, letter_to_state, reverse_alphabet = reconstruct_discrete_traits(params.tree, leaf_to_attr, missing_data=params.missing_data,
+            pc=params.pc, sampling_bias_correction=params.sampling_bias_correction, verbose=params.verbose)
 
+    unique_states = sorted(letter_to_state.values())
     ###########################################################################
     ### output
     ###########################################################################
@@ -732,11 +772,11 @@ def mugration(params):
         ofile.write('Character to attribute mapping:\n')
         for state in unique_states:
             ofile.write('  %s: %s\n'%(reverse_alphabet[state], state))
-        ofile.write('\n\n'+str(treeanc.gtr)+'\n')
+        ofile.write('\n\n'+str(mug.gtr)+'\n')
         print("\nSaved inferred mugration model as:", gtr_name)
 
     terminal_count = 0
-    for n in treeanc.tree.find_clades():
+    for n in mug.tree.find_clades():
         if n.up is None:
             continue
         n.confidence=None
@@ -751,14 +791,14 @@ def mugration(params):
         conf_name = basename+'confidence.csv'
         with open(conf_name, 'w') as ofile:
             ofile.write('#name, '+', '.join(unique_states)+'\n')
-            for n in treeanc.tree.find_clades():
+            for n in mug.tree.find_clades():
                 ofile.write(n.name + ', '+', '.join([str(x) for x in n.marginal_profile[0]])+'\n')
         print("Saved table with ancestral state confidences as:", conf_name)
 
     # write tree to file
     outtree_name = basename+'annotated_tree.nexus'
-    Phylo.write(treeanc.tree, outtree_name, 'nexus')
-    print("Saved annotated tree as:",outtree_name)
+    Phylo.write(mug.tree, outtree_name, 'nexus')
+    print("Saved annotated tree as:", outtree_name)
 
     return 0
 
@@ -786,17 +826,22 @@ def estimate_clock_model(params):
     ###########################################################################
     ### ESTIMATE ROOT (if requested) AND DETERMINE TEMPORAL SIGNAL
     ###########################################################################
+    if params.aln is None and params.sequence_length is None:
+        print("one of arguments '--aln' and '--sequence-length' is required.", file=sys.stderr)
+        return 1
+
     basename = get_basename(params, outdir)
     myTree = TreeTime(dates=dates, tree=params.tree, aln=aln, gtr='JC69',
                       verbose=params.verbose, seq_len=params.sequence_length,
                       ref=ref)
+    myTree.tip_slack=params.tip_slack
     if myTree.tree is None:
         print("ERROR: tree loading failed. exiting...")
         return 1
 
     if params.clock_filter:
         n_bad = [n.name for n in myTree.tree.get_terminals() if n.bad_branch]
-        myTree.clock_filter(n_iqd=params.clock_filter)
+        myTree.clock_filter(n_iqd=params.clock_filter, reroot=params.reroot or 'least-squares')
         n_bad_after = [n.name for n in myTree.tree.get_terminals() if n.bad_branch]
         if len(n_bad_after)>len(n_bad):
             print("The following leaves don't follow a loose clock and "
@@ -805,19 +850,20 @@ def estimate_clock_model(params):
 
     if not params.keep_root:
         # reroot to optimal root, this assigns clock_model to myTree
-        if params.reroot in ['ML','best']:
-            myTree.run(root="least-squares", max_iter=0)
+        if params.covariation: # this requires branch length estimates
+            myTree.run(root="least-squares", max_iter=0,
+                       use_covariation=params.covariation)
 
-        res = myTree.reroot("least-squares",
+        res = myTree.reroot(params.reroot,
                       force_positive=not params.allow_negative_rate)
-        myTree.get_clock_model(covariation=(params.reroot!='least-squares'))
+        myTree.get_clock_model(covariation=params.covariation)
 
         if res==ttconf.ERROR:
             print("ERROR: unknown root or rooting mechanism!\n"
                   "\tvalid choices are 'least-squares', 'ML', and 'ML-rough'")
             return 1
     else:
-        myTree.get_clock_model(covariation=True)
+        myTree.get_clock_model(covariation=params.covariation)
 
     d2d = utils.DateConversion.from_regression(myTree.clock_model)
     print('\n',d2d)
@@ -831,7 +877,7 @@ def estimate_clock_model(params):
           '\nNegative rates suggest an inappropriate root.\n')
 
     print('\nThe estimated rate and tree correspond to a root date:')
-    if params.reroot in ['ML', 'best', 'ML-rough']:
+    if params.covariation:
         reg = myTree.clock_model
         dp = np.array([reg['intercept']/reg['slope']**2,-1./reg['slope']])
         droot = np.sqrt(reg['cov'][:2,:2].dot(dp).dot(dp))
